//  *****************************************************************************
//   Program:  WaspARM Monitor
//
//   Program:  main.c				Version:  0.89
//
//   Demonstration program for WaspARM Embedded Processor AT91SAM7S32
//
//   The WaspARM has optional external serial flash, 3-axis accelerometer,
//   16 bit DAC and 16 bit ADC>
//
//  Author:  BR and SS
//  Last update October 25, 2009
//  *****************************************************************************
/* Copyright 2009, SOC ROBOTICS, Inc.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holders nor the names of
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 * 4. Redistribution is only allowed in combination with SOC ROBOTICS,
 *    Inc. hardware products.
 *
 * THIS SOFTWARE IS PROVIDED BY SOC ROBOTICS, INC. AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL SOC ROBOTICS,
 * INC. OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * For additional information see http://www.soc-robotics.com
 */
//  *******************************************************
//                Header Files
//  *******************************************************
#include "AT91SAM7S32.h"
#include "board.h"
//#include "math.h"
#include "stdlib.h"
#include <stdio.h>
#include "string.h"
#include "ARM_Serial.h"
#include "Accelerometer.h"
#include "Wasp_Arm_Utilities.h"
#include "External_Converters.h"

//  *******************************************************
//                Function Prototypes
//  *******************************************************
void Timer0IrqHandler(void);
void FiqHandler(void);
//int putchar(char c);
//int getchar(void);

//  *******************************************************
//                External References
//  *******************************************************
extern	void LowLevelInit(void);
extern	void TimerSetup(void);
extern	unsigned enableIRQ(void);
extern	unsigned enableFIQ(void);

static volatile AT91PS_PMC pPMC = AT91C_BASE_PMC;// pointer to PMC data structure
static volatile AT91PS_PIO pPIO = AT91C_BASE_PIOA;// pointer to PIO data structure
static volatile AT91PS_SPI pSPI = AT91C_BASE_SPI;

//  *******************************************************
//               Global Variables
//  *******************************************************
unsigned char   SPIEnabled = NO;
unsigned int	FiqCount = 0;		// global uninitialized variable
unsigned int    SerialFlashStatus;

void LongDelay(int d)
{
	int j;
	for (d; d != 0; d--)
		for (j = 100000; j != 0; j-- );
}

//  Print Serial Flash Details
void SPI_GetManufacturerID(void)
{
	char text[20];
	unsigned int instatus,instatus2;

	if(SPIEnabled==NO) {
	    // Enable the SPI peripheral clock
	    pPMC->PMC_PCER = (1<<AT91C_ID_SPI);// enable the Serial Peripheral Interface clock
	    pPMC->PMC_PCER = (1<<AT91C_ID_PIOA);// enable the Parallel IO Controller clock

	    pPIO->PIO_ASR = (MOSI | SCK | SS);// PIO Peripheral A Select for MOSI, SCK and SS lines
	    pPIO->PIO_PDR = (MOSI | SCK | SS);// PIO Controller PIO Disable, enables peripheral control

	    pSPI->SPI_MR = AT91C_SPI_MSTR | AT91C_SPI_MODFDIS;//SPI Master, Disable Mode fault detection

	    //mode 0 or 3
	    pSPI->SPI_CSR[0] = AT91C_SPI_CSAAT | AT91C_SPI_NCPHA | AT91C_SPI_BITS_8 | AT91C_SPI_SCBR;//SPI Mode 0, Baud = MCK/0xFF

	    pSPI->SPI_CR = AT91C_SPI_SPIEN;//Enable the SPI controller

	    // Send Info Opcode
	    pSPI->SPI_TDR = 0x9F;
	    //wait until it starts to transfer
	    while(!(pSPI->SPI_SR & AT91C_SPI_TDRE));
	    pSPI->SPI_TDR = 0x00;
	    while(!(pSPI->SPI_SR & AT91C_SPI_RDRF));
	    instatus = pSPI->SPI_RDR;//rubbish data
	    while(!(pSPI->SPI_SR & AT91C_SPI_TDRE));
	    pSPI->SPI_TDR = 0x00;
	    while(!(pSPI->SPI_SR & AT91C_SPI_RDRF));
	    instatus = 0xFF & pSPI->SPI_RDR;//Manufacture ID
	    while(!(pSPI->SPI_SR & AT91C_SPI_TDRE));
	    pSPI->SPI_TDR = 0x00;
	    while(!(pSPI->SPI_SR & AT91C_SPI_RDRF));
	    instatus2 = 0xFF & pSPI->SPI_RDR;//Device ID
//     uart0_puts("Manufacture ID = ");
//     uart0_puts(Print_Int(instatus,text,HEX));
	    SerialFlashStatus = instatus2;
	    SPIEnabled = YES;
		}

     uart0_puts("External Serial Flash is ");
     switch(SerialFlashStatus)
     {
     	case 0x25:
     		uart0_puts("AT45DB081 - 8Mbit\n\r");
            break;
        case 0x26:
        	uart0_puts("AT45DB161 - 16Mbit\n\r");
            break;
        case 0x27:
        	uart0_puts("AT45DB321 - 32Mbit\n\r");
            break;
        case 0x28:
        	uart0_puts("AT45DB642 - 64Mbit\n\r");
            break;
        default:
        	uart0_puts("Unknown device size.  Code is -> ");
        	uart0_puts(Print_Int(instatus2,text,HEX));
        	uart0_puts("\n\r");
     }
}

void test_ADC()
{
	char mytext[13];
	ADC_Init();
	uart0_puts(Print_Int(0, mytext, DEC));
	uart0_puts("\n\r");
	uart0_puts(Print_Int(100000, mytext, DEC));
	uart0_puts("\n\r");
	uart0_puts(Print_Int(-9999, mytext, DEC));
		uart0_puts("\n\r");

	while(1) {
		Read_ADC();
		//itoa(123, mytext, 10);

		Print_Int(ad[5], mytext, DEC);
		uart0_puts(mytext);
		uart0_puts("\n\r");
		LongDelay(10);
		}
}

void Read_Acc()
{
	char mytext[13];
	char count = 0;

	uart0_puts("\n\rHello\n\r");

	ADC_Init();

	setAccelerometerPorts();
	setAccelerometerRange(ACCELEROMETER_1g5);
	enableAccelerometer(ON);

	uart0_puts("\n\ragain\n\r");

	while(count < 20)
	{
		uart0_putc('\f');//signal a formfeed (newpage)

		readAccelerometer();

		printvalue(X_AXIS, mytext);
		uart0_puts("X Axis reads : ");
		uart0_puts(mytext);

		printvalue(Y_AXIS, mytext);
		uart0_puts("\n\rY Axis reads : ");
		uart0_puts(mytext);

		printvalue(Z_AXIS, mytext);
		uart0_puts("\n\rZ Axis reads : ");
		uart0_puts(mytext);

		LongDelay(1);
		count++;

	}
	uart0_puts("\n\rSettle the board\n\r");

	LongDelay(5);

	uart0_puts("\n\rStarting my zero offset correction\n\r");
	zeroOffset();
	uart0_puts("Finished the correction\n\r");

	readAccelerometer();

	printvalue(X_AXIS, mytext);
	uart0_puts("\n\rX Axis reads : ");
	uart0_puts(mytext);

	printvalue(Y_AXIS, mytext);
	uart0_puts("\n\rY Axis reads : ");
	uart0_puts(mytext);

	printvalue(Z_AXIS, mytext);
	uart0_puts("\n\rZ Axis reads : ");
	uart0_puts(mytext);

	LongDelay(10);

	while(1) {
		readAccelerometer();

		// Signal a formfeed (newpage)
		uart0_putc('\f');

		printvalue(X_AXIS, mytext);
		uart0_puts("X Axis reads : ");
		uart0_puts(mytext);

		printvalue(Y_AXIS, mytext);
		uart0_puts("\n\rY Axis reads : ");
		uart0_puts(mytext);

		printvalue(Z_AXIS, mytext);
		uart0_puts("\n\rZ Axis reads : ");
		uart0_puts(mytext);

		LongDelay(1);
		}
}

void Test_E_DAC()
{
	int i = 0;
	InitDAC();

	//void ReleaseDAC(void);

	while(1) {
		WriteDAC(0x0);;

		LongDelay(1);

		WriteDAC(0xFFFF);

		LongDelay(1);
		}
}

void Test_E_ADC()
{
	char mytext[13];
	InitDAC();

	// Continuous conversion 15 samples per second
	if(InitI2CADC((0<<SC)|(3<<DR))){
		uart0_puts("Error, I2C device not responding\n\r");
		}
	else {
		uart0_puts("I2C device responded\n\r");
		}

	while(1) 	{
		WriteDAC(ReadI2CADC());
		}

	while(0) 	{
		Print_Int(ReadI2CADC(), mytext, DEC);
			uart0_puts(mytext);
			uart0_puts("\n\r");
		}
}

void Signon()
{
	uart0_puts("\n\rWaspARM AT91SAM7S32 Monitor Data Logger V0.89\n\r");
	uart0_puts("(c) Copyright 2009, SOC Robotics, Inc.\n\r");
	SPI_GetManufacturerID();
	uart0_puts("\n\r  s - Signon message\n\r");
	uart0_puts("  a - Output Analog data\n\r");
	uart0_puts("  c - Calibrate accelerometer\n\r");
	uart0_puts("  g - Read accelerometers\n\r");
	uart0_puts("  r - Output accelerometer data continuously\n\r\n\r");
}

#define ON  1
#define OFF 0

void  LedState(unsigned char  code) {
	volatile AT91PS_PIO		pPIO = AT91C_BASE_PIOA;			// pointer to PIO register structure

	if(code==ON)  pPIO->PIO_CODR = LED1;							// turn LED1 (DS1) on
	if(code==OFF) pPIO->PIO_SODR = LED1;							// turn LED1 (DS1) off
}

void PrintChannelId(unsigned char channelid)
{
	if(channelid==0) uart0_puts("  Yg   ");
	if(channelid==1) uart0_puts("  Zg   ");
	if(channelid==2) uart0_puts("      ");
	if(channelid==3) uart0_puts("      ");
	if(channelid==4) uart0_puts("  C gyro    ");
	if(channelid==5) uart0_puts("  Xg   ");
	if(channelid==6) uart0_puts("  B gyro   ");
	if(channelid==7) uart0_puts("  A gyro   ");
}

int	main (void) {
	char text[20],inchar;

	// Initialize the Atmel AT91SAM7S32 (watchdog, PLL clock, default interrupts, etc.)
	LowLevelInit();

	// enable the Timer0 peripheral clock
	volatile AT91PS_PMC	pPMC = AT91C_BASE_PMC;			// pointer to PMC data structure
	pPMC->PMC_PCER = (1<<AT91C_ID_TC0);					// enable Timer0 peripheral clock

	// Set up the LEDs
	volatile AT91PS_PIO	pPIO = AT91C_BASE_PIOA;			// pointer to PIO data structure
	pPIO->PIO_PER = LED_MASK;							// PIO Enable Register - allow PIO to control pin P0
	pPIO->PIO_OER = LED_MASK;							// PIO Output Enable Register - sets pin P0 output
	pPIO->PIO_SODR = LED_MASK;							// PIO Set Output Data Register - turns off the LED

	// Set up the AIC  registers for Timer 0
	volatile AT91PS_AIC	pAIC = AT91C_BASE_AIC;			// pointer to AIC data structure
	pAIC->AIC_IDCR = (1<<AT91C_ID_TC0);					// Disable timer 0 interrupt in AIC Interrupt Disable Command Register
	pAIC->AIC_SVR[AT91C_ID_TC0] =						// Set the TC0 IRQ handler address in AIC Source
    	(unsigned int)Timer0IrqHandler;        			// Vector Register[12]
	pAIC->AIC_SMR[AT91C_ID_TC0] =						// Set the interrupt source type and priority
   		(AT91C_AIC_SRCTYPE_INT_HIGH_LEVEL | 0x4 ); 		// in AIC Source Mode Register[12]
	pAIC->AIC_ICCR = (1<<AT91C_ID_TC0); 				// Clear the TC0 interrupt in AIC Interrupt Clear Command Register
	pAIC->AIC_IDCR = (0<<AT91C_ID_TC0);					// Remove disable timer 0 interrupt in AIC Interrupt Disable Command Reg
	pAIC->AIC_IECR = (1<<AT91C_ID_TC0); 				// Enable the TC0 interrupt in AIC Interrupt Enable Command Register

	// Set up the AIC registers for FIQ (pushbutton SW1)
	pAIC->AIC_IDCR = (1<<AT91C_ID_FIQ);					// Disable FIQ interrupt in AIC Interrupt Disable Command Register
	pAIC->AIC_SMR[AT91C_ID_FIQ] =						// Set the interrupt source type in AIC Source
    	(AT91C_AIC_SRCTYPE_INT_POSITIVE_EDGE);  		// Mode Register[0]
	pAIC->AIC_ICCR = (1<<AT91C_ID_FIQ); 				// Clear the FIQ interrupt in AIC Interrupt Clear Command Register
	pAIC->AIC_IDCR = (0<<AT91C_ID_FIQ);					// Remove disable FIQ interrupt in AIC Interrupt Disable Command Register
	pAIC->AIC_IECR = (1<<AT91C_ID_FIQ); 				// Enable the FIQ interrupt in AIC Interrupt Enable Command Register

	// Setup timer0 to generate a 10 msec periodic interrupt
	TimerSetup();

	// enable interrupts
	enableIRQ();
	enableFIQ();

	uart0_init ();
	LongDelay(1);

	ADC_Init();

	// Leave time for USB10 to come up
	LongDelay(120);

	Signon();
//    while(1) uart0_putc(0x55);

	setAccelerometerPorts();
	setAccelerometerRange(ACCELEROMETER_4g);
	enableAccelerometer(ON);

	int i = 0;

	blinker(2);

	//setAccelerometerRange(ACCELEROMETER_6g);
	//LongDelay(7);

	uart0_putc('>');
	while(1) {
		if(uart0_kbhit()) {
			inchar = uart0_getc();
			uart0_putc(inchar);

			switch (inchar) {
				case 'c':
					LedState(ON);
					uart0_puts("\n\rCalibrating Accelerometer\n\r");
					uart0_puts("Hold Still\n\r");
					LongDelay(40);
					zeroOffset();
					uart0_puts("Done");
					uart0_puts("\n\r>");
					break;
				case 'a':
					LedState(OFF);
					uart0_puts("\n\rOutput Data from all 8 Analog Inputs\n\r");
					Read_ADC();
					for(i=0; i<8; i++) {
						uart0_puts("Ch ");
						uart0_putc(0x30+i);
						uart0_puts(" = ");
						Print_Int(ad[i], text, DEC);
						uart0_puts(text);
						PrintChannelId(i);
						uart0_puts("\n\r");
						}
					uart0_putc('>');
					break;
				case 'r':
					inchar = 0x08;
					uart0_puts("\n\rStart output of accelerometer values continuously\n\r");
					uart0_puts("Hit any key to stop\n\r");
					while(inchar == 0x08) {
						if(uart0_kbhit()) inchar = uart0_getc();
						else inchar = 0x08;

						readAccelerometer();
						uart0_puts("\rX Y Z Axis = ");
						printvalue(X_AXIS, text);
						uart0_puts(text);
						uart0_puts("  ");
						printvalue(Y_AXIS, text);
						uart0_puts(text);
						uart0_puts("  ");
						printvalue(Z_AXIS, text);
						uart0_puts(text);
						uart0_puts("                   ");
						uart0_putc('\r');
						LongDelay(7);
						}
					uart0_puts("\n\r>");
					break;
				case 'g':
					readAccelerometer();
					uart0_puts("\n\rX Y Z Axis = ");
					printvalue(X_AXIS, text);
					uart0_puts(text);
					uart0_puts("  ");
					printvalue(Y_AXIS, text);
					uart0_puts(text);
					uart0_puts("  ");
					printvalue(Z_AXIS, text);
					uart0_puts(text);
					uart0_puts("                          \r");
					uart0_puts("\n\r>");
					break;
				case 's':
					Signon();
					uart0_puts("\n\r>");
					break;
				default:
					uart0_puts("\n\r>");
					break;
				}
			}
		else inchar = 0x08;

		blinker(1);
	    }
}
